@@ -619,6 +619,75 @@ def detect_github_repo(loglines):
     return None
 
 
+def extract_repo_from_session(session):
+    """Extract GitHub repo from session metadata.
+
+    Looks in session_context.outcomes for git_info.repo,
+    or parses from session_context.sources URL.
+
+    Returns repo as "owner/name" or None.
+    """
+    context = session.get("session_context", {})
+
+    # Try outcomes first (has clean repo format)
+    outcomes = context.get("outcomes", [])
+    for outcome in outcomes:
+        if outcome.get("type") == "git_repository":
+            git_info = outcome.get("git_info", {})
+            repo = git_info.get("repo")
+            if repo:
+                return repo
+
+    # Fall back to sources URL
+    sources = context.get("sources", [])
+    for source in sources:
+        if source.get("type") == "git_repository":
+            url = source.get("url", "")
+            # Parse github.com/owner/repo from URL
+            if "github.com/" in url:
+                # Extract owner/repo from https://github.com/owner/repo
+                match = re.search(r"github\.com/([^/]+/[^/]+?)(?:\.git)?$", url)
+                if match:
+                    return match.group(1)
+
+    return None
+
+
+def enrich_sessions_with_repos(sessions, token=None, org_uuid=None, fetch_fn=None):
+    """Enrich sessions with repo information from session metadata.
+
+    Args:
+        sessions: List of session dicts from the API
+        token: Unused (kept for backward compatibility)
+        org_uuid: Unused (kept for backward compatibility)
+        fetch_fn: Unused (kept for backward compatibility)
+
+    Returns:
+        List of session dicts with 'repo' key added
+    """
+    enriched = []
+    for session in sessions:
+        session_copy = dict(session)
+        session_copy["repo"] = extract_repo_from_session(session)
+        enriched.append(session_copy)
+    return enriched
+
+
+def filter_sessions_by_repo(sessions, repo):
+    """Filter sessions by repo.
+
+    Args:
+        sessions: List of session dicts with 'repo' key
+        repo: Repo to filter by (owner/name), or None to return all
+
+    Returns:
+        Filtered list of sessions
+    """
+    if repo is None:
+        return sessions
+    return [s for s in sessions if s.get("repo") == repo]
+
+
 def format_json(obj):
     try:
         if isinstance(obj, str):
@@ -708,6 +777,7 @@ def render_content_block(block):
     elif block_type == "tool_result":
         content = block.get("content", "")
         is_error = block.get("is_error", False)
+        has_images = False
 
         # Check for git commits and render with styled cards
         if isinstance(content, str):
@@ -737,11 +807,35 @@ def render_content_block(block):
                 content_html = "".join(parts)
             else:
                 content_html = f"<pre>{html.escape(content)}</pre>"
-        elif isinstance(content, list) or is_json_like(content):
+        elif isinstance(content, list):
+            # Handle tool result content that contains multiple blocks (text, images, etc.)
+            parts = []
+            for item in content:
+                if isinstance(item, dict):
+                    item_type = item.get("type", "")
+                    if item_type == "text":
+                        text = item.get("text", "")
+                        if text:
+                            parts.append(f"<pre>{html.escape(text)}</pre>")
+                    elif item_type == "image":
+                        source = item.get("source", {})
+                        media_type = source.get("media_type", "image/png")
+                        data = source.get("data", "")
+                        if data:
+                            parts.append(_macros.image_block(media_type, data))
+                            has_images = True
+                    else:
+                        # Unknown type, render as JSON
+                        parts.append(format_json(item))
+                else:
+                    # Non-dict item, escape as text
+                    parts.append(f"<pre>{html.escape(str(item))}</pre>")
+            content_html = "".join(parts) if parts else format_json(content)
+        elif is_json_like(content):
             content_html = format_json(content)
         else:
             content_html = format_json(content)
-        return _macros.tool_result(content_html, is_error)
+        return _macros.tool_result(content_html, is_error, has_images)
     else:
         return format_json(block)
 
@@ -1666,15 +1760,19 @@ def resolve_credentials(token, org_uuid):
 def format_session_for_display(session_data):
     """Format a session for display in the list or picker.
 
+    Shows repo first (if available), then date, then title.
     Returns a formatted string.
     """
-    session_id = session_data.get("id", "unknown")
     title = session_data.get("title", "Untitled")
     created_at = session_data.get("created_at", "")
+    repo = session_data.get("repo")
     # Truncate title if too long
-    if len(title) > 60:
-        title = title[:57] + "..."
-    return f"{session_id}  {created_at[:19] if created_at else 'N/A':19}  {title}"
+    if len(title) > 50:
+        title = title[:47] + "..."
+    # Format: repo (or placeholder)  date  title
+    repo_display = repo if repo else "(no repo)"
+    date_display = created_at[:19] if created_at else "N/A"
+    return f"{repo_display:30}  {date_display:19}  {title}"
 
 
 def generate_html_from_session_data(session_data, output_dir, github_repo=None):
@@ -1866,7 +1964,7 @@ def generate_html_from_session_data(session_data, output_dir, github_repo=None):
 )
 @click.option(
     "--repo",
-    help="GitHub repo (owner/name) for commit links. Auto-detected from git push output if not specified.",
+    help="GitHub repo (owner/name). Filters session list and sets default for commit links.",
 )
 @click.option(
     "--gist",
@@ -1920,16 +2018,20 @@ def web_cmd(
         if not sessions:
             raise click.ClickException("No sessions found.")
 
+        # Enrich sessions with repo information (extracted from session metadata)
+        sessions = enrich_sessions_with_repos(sessions)
+
+        # Filter by repo if specified
+        if repo:
+            sessions = filter_sessions_by_repo(sessions, repo)
+            if not sessions:
+                raise click.ClickException(f"No sessions found for repo: {repo}")
+
         # Build choices for questionary
         choices = []
         for s in sessions:
             sid = s.get("id", "unknown")
-            title = s.get("title", "Untitled")
-            created_at = s.get("created_at", "")
-            # Truncate title if too long
-            if len(title) > 50:
-                title = title[:47] + "..."
-            display = f"{created_at[:19] if created_at else 'N/A':19}  {title}"
+            display = format_session_for_display(s)
             choices.append(questionary.Choice(title=display, value=sid))
 
         selected = questionary.select(
